@@ -14,17 +14,27 @@
 
 import operator as op
 import unittest
+import warnings
 
 import mock
 import six
 
+try:
+    import pyarrow
+except ImportError:  # pragma: NO COVER
+    pyarrow = None
+
 from google.api_core import exceptions
 
 try:
+    from google.cloud import bigquery_storage_v1
     from google.cloud import bigquery_storage_v1beta1
 except ImportError:  # pragma: NO COVER
+    bigquery_storage_v1 = None
     bigquery_storage_v1beta1 = None
 
+from tests.unit.helpers import _to_pyarrow
+
 
 class TestCursor(unittest.TestCase):
     @staticmethod
@@ -36,7 +46,15 @@ class TestCursor(unittest.TestCase):
     def _make_one(self, *args, **kw):
         return self._get_target_class()(*args, **kw)
 
-    def _mock_client(self, rows=None, schema=None, num_dml_affected_rows=None):
+    def _mock_client(
+        self,
+        rows=None,
+        schema=None,
+        num_dml_affected_rows=None,
+        default_query_job_config=None,
+        dry_run_job=False,
+        total_bytes_processed=0,
+    ):
         from google.cloud.bigquery import client
 
         if rows is None:
@@ -49,25 +67,44 @@ class TestCursor(unittest.TestCase):
             total_rows=total_rows,
             schema=schema,
             num_dml_affected_rows=num_dml_affected_rows,
+            dry_run=dry_run_job,
+            total_bytes_processed=total_bytes_processed,
         )
         mock_client.list_rows.return_value = rows
+        mock_client._default_query_job_config = default_query_job_config
+
+        # Assure that the REST client gets used, not the BQ Storage client.
+        mock_client._create_bqstorage_client.return_value = None
+
         return mock_client
 
-    def _mock_bqstorage_client(self, rows=None, stream_count=0):
-        from google.cloud.bigquery_storage_v1beta1 import client
-        from google.cloud.bigquery_storage_v1beta1 import types
+    def _mock_bqstorage_client(self, rows=None, stream_count=0, v1beta1=False):
+        from google.cloud.bigquery_storage_v1 import client
+        from google.cloud.bigquery_storage_v1 import types
+        from google.cloud.bigquery_storage_v1beta1 import types as types_v1beta1
 
         if rows is None:
             rows = []
 
-        mock_client = mock.create_autospec(client.BigQueryStorageClient)
+        if v1beta1:
+            mock_client = mock.create_autospec(
+                bigquery_storage_v1beta1.BigQueryStorageClient
+            )
+            mock_read_session = mock.MagicMock(
+                streams=[
+                    types_v1beta1.Stream(name="streams/stream_{}".format(i))
+                    for i in range(stream_count)
+                ]
+            )
+        else:
+            mock_client = mock.create_autospec(client.BigQueryReadClient)
+            mock_read_session = mock.MagicMock(
+                streams=[
+                    types.ReadStream(name="streams/stream_{}".format(i))
+                    for i in range(stream_count)
+                ]
+            )
 
-        mock_read_session = mock.MagicMock(
-            streams=[
-                types.Stream(name="streams/stream_{}".format(i))
-                for i in range(stream_count)
-            ]
-        )
         mock_client.create_read_session.return_value = mock_read_session
 
         mock_rows_stream = mock.MagicMock()
@@ -76,18 +113,34 @@ class TestCursor(unittest.TestCase):
 
         return mock_client
 
-    def _mock_job(self, total_rows=0, schema=None, num_dml_affected_rows=None):
+    def _mock_job(
+        self,
+        total_rows=0,
+        schema=None,
+        num_dml_affected_rows=None,
+        dry_run=False,
+        total_bytes_processed=0,
+    ):
         from google.cloud.bigquery import job
 
         mock_job = mock.create_autospec(job.QueryJob)
         mock_job.error_result = None
         mock_job.state = "DONE"
-        mock_job.result.return_value = mock_job
-        mock_job._query_results = self._mock_results(
-            total_rows=total_rows,
-            schema=schema,
-            num_dml_affected_rows=num_dml_affected_rows,
-        )
+        mock_job.dry_run = dry_run
+
+        if dry_run:
+            mock_job.result.side_effect = exceptions.NotFound
+            mock_job.total_bytes_processed = total_bytes_processed
+        else:
+            mock_job.result.return_value = mock_job
+            mock_job._query_results = self._mock_results(
+                total_rows=total_rows,
+                schema=schema,
+                num_dml_affected_rows=num_dml_affected_rows,
+            )
+            mock_job.destination.to_bqstorage.return_value = (
+                "projects/P/datasets/DS/tables/T"
+            )
 
         if num_dml_affected_rows is None:
             mock_job.statement_type = None  # API sends back None for SELECT
@@ -122,6 +175,31 @@ class TestCursor(unittest.TestCase):
         # close() is a no-op, there is nothing to test.
         cursor.close()
 
+    def test_raises_error_if_closed(self):
+        from google.cloud.bigquery.dbapi import connect
+        from google.cloud.bigquery.dbapi.exceptions import ProgrammingError
+
+        connection = connect(self._mock_client())
+        cursor = connection.cursor()
+        cursor.close()
+
+        method_names = (
+            "close",
+            "execute",
+            "executemany",
+            "fetchall",
+            "fetchmany",
+            "fetchone",
+            "setinputsizes",
+            "setoutputsize",
+        )
+
+        for method in method_names:
+            with six.assertRaisesRegex(
+                self, ProgrammingError, r"Operating on a closed cursor\."
+            ):
+                getattr(cursor, method)()
+
     def test_fetchone_wo_execute_raises_error(self):
         from google.cloud.bigquery import dbapi
 
@@ -213,8 +291,9 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(rows[0], (1,))
 
     @unittest.skipIf(
-        bigquery_storage_v1beta1 is None, "Requires `google-cloud-bigquery-storage`"
+        bigquery_storage_v1 is None, "Requires `google-cloud-bigquery-storage`"
     )
+    @unittest.skipIf(pyarrow is None, "Requires `pyarrow`")
     def test_fetchall_w_bqstorage_client_fetch_success(self):
         from google.cloud.bigquery import dbapi
         from google.cloud.bigquery import table
@@ -225,8 +304,18 @@ class TestCursor(unittest.TestCase):
             table.Row([2.4, 2.1, 2.3, 2.2], {"bar": 3, "baz": 2, "foo": 1, "quux": 0}),
         ]
         bqstorage_streamed_rows = [
-            {"bar": 1.2, "foo": 1.1, "quux": 1.4, "baz": 1.3},
-            {"bar": 2.2, "foo": 2.1, "quux": 2.4, "baz": 2.3},
+            {
+                "bar": _to_pyarrow(1.2),
+                "foo": _to_pyarrow(1.1),
+                "quux": _to_pyarrow(1.4),
+                "baz": _to_pyarrow(1.3),
+            },
+            {
+                "bar": _to_pyarrow(2.2),
+                "foo": _to_pyarrow(2.1),
+                "quux": _to_pyarrow(2.4),
+                "baz": _to_pyarrow(2.3),
+            },
         ]
 
         mock_client = self._mock_client(rows=row_data)
@@ -258,6 +347,70 @@ class TestCursor(unittest.TestCase):
     @unittest.skipIf(
         bigquery_storage_v1beta1 is None, "Requires `google-cloud-bigquery-storage`"
     )
+    @unittest.skipIf(pyarrow is None, "Requires `pyarrow`")
+    def test_fetchall_w_bqstorage_client_v1beta1_fetch_success(self):
+        from google.cloud.bigquery import dbapi
+        from google.cloud.bigquery import table
+
+        # use unordered data to also test any non-determenistic key order in dicts
+        row_data = [
+            table.Row([1.4, 1.1, 1.3, 1.2], {"bar": 3, "baz": 2, "foo": 1, "quux": 0}),
+            table.Row([2.4, 2.1, 2.3, 2.2], {"bar": 3, "baz": 2, "foo": 1, "quux": 0}),
+        ]
+        bqstorage_streamed_rows = [
+            {
+                "bar": _to_pyarrow(1.2),
+                "foo": _to_pyarrow(1.1),
+                "quux": _to_pyarrow(1.4),
+                "baz": _to_pyarrow(1.3),
+            },
+            {
+                "bar": _to_pyarrow(2.2),
+                "foo": _to_pyarrow(2.1),
+                "quux": _to_pyarrow(2.4),
+                "baz": _to_pyarrow(2.3),
+            },
+        ]
+
+        mock_client = self._mock_client(rows=row_data)
+        mock_bqstorage_client = self._mock_bqstorage_client(
+            stream_count=1, rows=bqstorage_streamed_rows, v1beta1=True
+        )
+
+        connection = dbapi.connect(
+            client=mock_client, bqstorage_client=mock_bqstorage_client,
+        )
+        cursor = connection.cursor()
+        cursor.execute("SELECT foo, bar FROM some_table")
+
+        with warnings.catch_warnings(record=True) as warned:
+            rows = cursor.fetchall()
+
+        # a deprecation warning should have been emitted
+        expected_warnings = [
+            warning
+            for warning in warned
+            if issubclass(warning.category, DeprecationWarning)
+            and "v1beta1" in str(warning)
+        ]
+        self.assertEqual(len(expected_warnings), 1, "Deprecation warning not raised.")
+
+        # the default client was not used
+        mock_client.list_rows.assert_not_called()
+
+        # check the data returned
+        field_value = op.itemgetter(1)
+        sorted_row_data = [sorted(row.items(), key=field_value) for row in rows]
+        expected_row_data = [
+            [("foo", 1.1), ("bar", 1.2), ("baz", 1.3), ("quux", 1.4)],
+            [("foo", 2.1), ("bar", 2.2), ("baz", 2.3), ("quux", 2.4)],
+        ]
+
+        self.assertEqual(sorted_row_data, expected_row_data)
+
+    @unittest.skipIf(
+        bigquery_storage_v1 is None, "Requires `google-cloud-bigquery-storage`"
+    )
     def test_fetchall_w_bqstorage_client_fetch_no_rows(self):
         from google.cloud.bigquery import dbapi
 
@@ -279,7 +432,7 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(rows, [])
 
     @unittest.skipIf(
-        bigquery_storage_v1beta1 is None, "Requires `google-cloud-bigquery-storage`"
+        bigquery_storage_v1 is None, "Requires `google-cloud-bigquery-storage`"
     )
     def test_fetchall_w_bqstorage_client_fetch_error_no_fallback(self):
         from google.cloud.bigquery import dbapi
@@ -306,62 +459,6 @@ class TestCursor(unittest.TestCase):
         # the default client was not used
         mock_client.list_rows.assert_not_called()
 
-    @unittest.skipIf(
-        bigquery_storage_v1beta1 is None, "Requires `google-cloud-bigquery-storage`"
-    )
-    def test_fetchall_w_bqstorage_client_fetch_error_fallback_on_client(self):
-        from google.cloud.bigquery import dbapi
-        from google.cloud.bigquery import table
-
-        # use unordered data to also test any non-determenistic key order in dicts
-        row_data = [
-            table.Row([1.4, 1.1, 1.3, 1.2], {"bar": 3, "baz": 2, "foo": 1, "quux": 0}),
-            table.Row([2.4, 2.1, 2.3, 2.2], {"bar": 3, "baz": 2, "foo": 1, "quux": 0}),
-        ]
-        bqstorage_streamed_rows = [
-            {"bar": 1.2, "foo": 1.1, "quux": 1.4, "baz": 1.3},
-            {"bar": 2.2, "foo": 2.1, "quux": 2.4, "baz": 2.3},
-        ]
-
-        mock_client = self._mock_client(rows=row_data)
-        mock_bqstorage_client = self._mock_bqstorage_client(
-            stream_count=1, rows=bqstorage_streamed_rows,
-        )
-        request_error = exceptions.BadRequest("BQ storage what??")
-        mock_bqstorage_client.create_read_session.side_effect = request_error
-
-        connection = dbapi.connect(
-            client=mock_client, bqstorage_client=mock_bqstorage_client,
-        )
-        cursor = connection.cursor()
-        cursor.execute("SELECT foo, bar FROM some_table")
-
-        logger_patcher = mock.patch("google.cloud.bigquery.dbapi.cursor._LOGGER")
-        with logger_patcher as mock_logger:
-            rows = cursor.fetchall()
-
-        # both client were used
-        mock_bqstorage_client.create_read_session.assert_called()
-        mock_client.list_rows.assert_called()
-
-        # fallback to default API should have been logged
-        relevant_calls = [
-            call
-            for call in mock_logger.debug.call_args_list
-            if call.args and "tabledata.list API" in call.args[0]
-        ]
-        self.assertTrue(relevant_calls)
-
-        # check the data returned
-        field_value = op.itemgetter(1)
-        sorted_row_data = [sorted(row.items(), key=field_value) for row in rows]
-        expected_row_data = [
-            [("foo", 1.1), ("bar", 1.2), ("baz", 1.3), ("quux", 1.4)],
-            [("foo", 2.1), ("bar", 2.2), ("baz", 2.3), ("quux", 2.4)],
-        ]
-
-        self.assertEqual(sorted_row_data, expected_row_data)
-
     def test_execute_custom_job_id(self):
         from google.cloud.bigquery.dbapi import connect
 
@@ -373,7 +470,27 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(args[0], "SELECT 1;")
         self.assertEqual(kwargs["job_id"], "foo")
 
-    def test_execute_custom_job_config(self):
+    def test_execute_w_default_config(self):
+        from google.cloud.bigquery.dbapi import connect
+        from google.cloud.bigquery import job
+
+        default_config = job.QueryJobConfig(use_legacy_sql=False, flatten_results=True)
+        client = self._mock_client(
+            rows=[], num_dml_affected_rows=0, default_query_job_config=default_config
+        )
+        connection = connect(client)
+        cursor = connection.cursor()
+
+        cursor.execute("SELECT 1;", job_id="foo")
+
+        _, kwargs = client.query.call_args
+        used_config = kwargs["job_config"]
+        expected_config = job.QueryJobConfig(
+            use_legacy_sql=False, flatten_results=True, query_parameters=[]
+        )
+        self.assertEqual(used_config._properties, expected_config._properties)
+
+    def test_execute_custom_job_config_wo_default_config(self):
         from google.cloud.bigquery.dbapi import connect
         from google.cloud.bigquery import job
 
@@ -387,6 +504,29 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(kwargs["job_id"], "foo")
         self.assertEqual(kwargs["job_config"], config)
 
+    def test_execute_custom_job_config_w_default_config(self):
+        from google.cloud.bigquery.dbapi import connect
+        from google.cloud.bigquery import job
+
+        default_config = job.QueryJobConfig(use_legacy_sql=False, flatten_results=True)
+        client = self._mock_client(
+            rows=[], num_dml_affected_rows=0, default_query_job_config=default_config
+        )
+        connection = connect(client)
+        cursor = connection.cursor()
+        config = job.QueryJobConfig(use_legacy_sql=True)
+
+        cursor.execute("SELECT 1;", job_id="foo", job_config=config)
+
+        _, kwargs = client.query.call_args
+        used_config = kwargs["job_config"]
+        expected_config = job.QueryJobConfig(
+            use_legacy_sql=True,  # the config passed to execute() prevails
+            flatten_results=True,  # from the default
+            query_parameters=[],
+        )
+        self.assertEqual(used_config._properties, expected_config._properties)
+
     def test_execute_w_dml(self):
         from google.cloud.bigquery.dbapi import connect
 
@@ -442,6 +582,35 @@ class TestCursor(unittest.TestCase):
         row = cursor.fetchone()
         self.assertIsNone(row)
 
+    def test_execute_w_query_dry_run(self):
+        from google.cloud.bigquery.job import QueryJobConfig
+        from google.cloud.bigquery.schema import SchemaField
+        from google.cloud.bigquery import dbapi
+
+        connection = dbapi.connect(
+            self._mock_client(
+                rows=[("hello", "world", 1), ("howdy", "y'all", 2)],
+                schema=[
+                    SchemaField("a", "STRING", mode="NULLABLE"),
+                    SchemaField("b", "STRING", mode="REQUIRED"),
+                    SchemaField("c", "INTEGER", mode="NULLABLE"),
+                ],
+                dry_run_job=True,
+                total_bytes_processed=12345,
+            )
+        )
+        cursor = connection.cursor()
+
+        cursor.execute(
+            "SELECT a, b, c FROM hello_world WHERE d > 3;",
+            job_config=QueryJobConfig(dry_run=True),
+        )
+
+        self.assertEqual(cursor.rowcount, 0)
+        self.assertIsNone(cursor.description)
+        rows = cursor.fetchall()
+        self.assertEqual(list(rows), [])
+
     def test_execute_raises_if_result_raises(self):
         import google.cloud.exceptions
 
@@ -451,8 +620,10 @@ class TestCursor(unittest.TestCase):
         from google.cloud.bigquery.dbapi import exceptions
 
         job = mock.create_autospec(job.QueryJob)
+        job.dry_run = None
         job.result.side_effect = google.cloud.exceptions.GoogleCloudError("")
         client = mock.create_autospec(client.Client)
+        client._default_query_job_config = None
         client.query.return_value = job
         connection = connect(client)
         cursor = connection.cursor()
